<?php

declare(strict_types=1);

namespace Drupal\entity_purge_commerce\Plugin\EntityPurge\Configurator;

use Drupal\entity_purge\Entity\PurgeTypeInterface;
use Drupal\entity_purge\Exception\InvalidConfigurationException;
use Drupal\entity_purge\Plugin\EntityPurge\Configurator\Age;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\Query\QueryInterface;

/**
 * Configurator that purges cart entities based on their age.
 *
 * The Commerce Cart module provides its own system to delete abandoned carts
 * via cron/queue. It is less configurable though, and it uses the `changed`
 * timestamp to determine whether a cart should be deleted. This is problematic
 * in many cases because a cart can be refreshed in a number of circumstances
 * without the user interacting with it, at which point the `changed` timestamp
 * gets updated - preventing it from being deleted even if it is actually
 * abandoned.
 *
 * With this plugin carts can be deleted based on the `created` timestamp.
 *
 * phpcs:disable
 * @EntityPurgeConfigurator(
 *   id = "commerce_cart_age",
 *   label = @Translation("Commerce cart age"),
 *   description = @Translation("Purges cart order entities based on their age as determined by a timestamp field."),
 * )
 * @I Support restricting entity types via the plugin definition
 *    type     : improvement
 *    priority : low
 *    labels   : configurator, validation
 * phpcs:enable
 * @I Validate that the `commerce_cart` module is enabled
 *    type     : improvement
 *    priority : low
 *    labels   : configurator, validation
 * phpcs:enable
 */
class CartAge extends Age {

  /**
   * {@inheritdoc}
   */
  public function allowEntityPurge(
    PurgeTypeInterface $purge_type,
    EntityInterface $entity,
  ): bool {
    if (!parent::allowEntityPurge($purge_type, $entity)) {
      return FALSE;
    }

    // The entity collection query must already be guaranteeing that only carts
    // reach purging, but due to the potentially catastrophic consequences of
    // deleting placed orders let's place another check to prevent programmatic
    // mistakes potentially made in query alterations or plugin
    // overrides/extensions.
    return empty($entity->get('cart')->first()->getValue()['value']) ?
      FALSE :
      TRUE;
  }

  /**
   * {@inheritdoc}
   */
  protected function validateConfiguration(): void {
    parent::validateConfiguration();

    if ($this->configuration['entity_type_id'] === 'commerce_order') {
      return;
    }

    throw new InvalidConfigurationException(sprintf(
      'The "%s" Entity Purge configurator can only be used on "commerce_order" entities, "%s" given.',
      $this->getPluginId(),
      $this->configuration['entity_type_id'],
    ));
  }

  /**
   * {@inheritdoc}
   */
  protected function buildQuery(): QueryInterface {
    $query = parent::buildQuery();
    $query->condition('cart', TRUE);

    return $query;
  }

}
