<?php

namespace Drupal\Tests\entity_purge_commerce\Unit\Plugin\EntityPurge\Configurator;

use Drupal\entity_purge\Entity\PurgeTypeInterface;
use Drupal\entity_purge\Exception\InvalidConfigurationException;
use Drupal\entity_purge_commerce\Plugin\EntityPurge\Configurator\CartAge;
use Drupal\Tests\entity_purge\Unit\Configurator\PluginBaseTestTrait;
use Drupal\Tests\entity_purge\Unit\Plugin\EntityPurge\Configurator\AgeTestTrait;

use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Tests\UnitTestCase;

/**
 * @coversDefaultClass \Drupal\entity_purge_commerce\Plugin\EntityPurge\Configurator\CartAge
 *
 * @group contrib
 * @group entity_purge
 * @group unit
 */
class CartAgeTest extends UnitTestCase {

  use PluginBaseTestTrait {
    testAllowEntityPurgeWithDebugging as allowEntityPurgeWithDebugging;
    testAllowEntityPurgeWithoutDebugging as allowEntityPurgeWithoutDebugging;
    testDefaultConfiguration as defaultConfiguration;
    testAllowEntityPurgeWithDefaults as allowEntityPurgeWithDefaults;
  }

  use AgeTestTrait {
    testConfigurationRequiredPropertiesEntityTypeId as requiredPropertiesEntityTypeId;
    testConfigurationRequiredPropertiesFieldName as requiredPropertiesFieldName;
    testConfigurationRequiredPropertiesMinAge as requiredPropertiesMinAge;
    testConfigurationRequiredPropertiesLimit as requiredPropertiesLimit;
    someRequiredConfiguration as traitSomeRequiredConfiguration;
  }

  /**
   * Tests that non-cart orders are not allowed to be purged.
   *
   * @covers ::allowEntityPurge
   */
  public function testAllowEntityPurgeNonCart() {
    $cart_age = new CartAge(
      $this->someRequiredConfiguration(),
      'test',
      [],
      ...$this->revealedExtraArguments(),
    );

    $purge_type = $this->prophesize(PurgeTypeInterface::class);

    $field_item = $this->prophesize(FieldItemInterface::class);
    $field_item
      ->getValue()
      ->willReturn(['value' => 0]);

    $field_item_list = $this->prophesize(FieldItemListInterface::class);
    $field_item_list
      ->first()
      ->willReturn($field_item->reveal());

    $entity = $this->prophesize(FieldableEntityInterface::class);
    $entity
      ->get('cart')
      ->shouldBeCalledOnce()
      ->willReturn($field_item_list->reveal());

    $this->assertEquals(
      FALSE,
      $cart_age->allowEntityPurge($purge_type->reveal(), $entity->reveal()),
    );
  }

  /**
   * Tests that cart orders are allowed to be purged.
   *
   * @covers ::allowEntityPurge
   */
  public function testAllowEntityPurgeCart() {
    $cart_age = new CartAge(
      $this->someRequiredConfiguration(),
      'test',
      [],
      ...$this->revealedExtraArguments(),
    );

    $purge_type = $this->prophesize(PurgeTypeInterface::class);

    $field_item = $this->prophesize(FieldItemInterface::class);
    $field_item
      ->getValue()
      ->willReturn(['value' => 1]);

    $field_item_list = $this->prophesize(FieldItemListInterface::class);
    $field_item_list
      ->first()
      ->willReturn($field_item->reveal());

    $entity = $this->prophesize(FieldableEntityInterface::class);
    $entity
      ->get('cart')
      ->shouldBeCalledOnce()
      ->willReturn($field_item_list->reveal());

    $this->assertEquals(
      TRUE,
      $cart_age->allowEntityPurge($purge_type->reveal(), $entity->reveal()),
    );
  }

  /**
   * Tests that the `entity_type_id` configuration is required.
   *
   * @covers ::validateConfiguration
   */
  public function testConfigurationRequiredPropertiesEntityTypeId() {
    $this->requiredPropertiesEntityTypeId(CartAge::class);
  }

  /**
   * Tests that the `field_name` configuration is required.
   *
   * @covers ::validateConfiguration
   */
  public function testConfigurationRequiredPropertiesFieldName() {
    $this->requiredPropertiesFieldName(CartAge::class);
  }

  /**
   * Tests that the `min_age` configuration is required.
   *
   * @covers ::validateConfiguration
   */
  public function testConfigurationRequiredPropertiesMinAge() {
    $this->requiredPropertiesMinAge(CartAge::class);
  }

  /**
   * Tests that the `limit` configuration is required.
   *
   * @covers ::validateConfiguration
   */
  public function testConfigurationRequiredPropertiesLimit() {
    $this->requiredPropertiesLimit(CartAge::class);
  }

  /**
   * Tests that the `entity_type_id` is required to be `commerce_order`.
   *
   * @covers ::validateConfiguration
   */
  public function testConfigurationEntityTypeIdIsCommerceOrder() {
    $configuration = $this->someRequiredConfiguration();
    $configuration['entity_type_id'] = 'node';

    $this->expectException(InvalidConfigurationException::class);
    new CartAge(
      $configuration,
      'test',
      [],
      ...$this->revealedExtraArguments(),
    );
  }

  /**
   * Tests that entities are not purged when debugging is enabled.
   *
   * @covers ::allowEntityPurge
   */
  public function testAllowEntityPurgeWithDebugging() {
    $this->allowEntityPurgeWithDebugging(
      CartAge::class,
      $this->someRequiredConfiguration(),
      $this->revealedExtraArguments(),
    );
  }

  /**
   * Tests the expected default configuration.
   *
   * @covers ::__construct
   */
  public function testDefaultConfiguration() {
    $this->defaultConfiguration();
  }

  /**
   * Returns legitimate values for the required configuration by the plugin.
   */
  protected function someRequiredConfiguration(): array {
    return ['entity_type_id' => 'commerce_order'] +
      $this->traitSomeRequiredConfiguration();
  }

}
