<?php

declare(strict_types=1);

namespace Drupal\entity_purge;

use Drupal\entity_purge\Entity\PurgeTypeInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\Exception\UndefinedLinkTemplateException;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Psr\Log\LoggerInterface;

/**
 * The default implementation of the purger service.
 */
class Purger implements PurgerInterface {

  use StringTranslationTrait;

  /**
   * Constructs a new Purger object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger to use.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected LoggerInterface $logger,
    TranslationInterface $string_translation,
  ) {
    // Injections required by traits.
    $this->stringTranslation = $string_translation;
  }

  /**
   * {@inheritdoc}
   */
  public function run(array $purge_types): void {
    foreach ($purge_types as $purge_type) {
      $purge_type->status() ? $this->doPurge($purge_type) : NULL;
    }
  }

  /**
   * Runs a purge operation for the given purge type.
   *
   * @param \Drupal\entity_purge\Entity\PurgeTypeInterface $purge_type
   *   The purge type for which to run the purge operation.
   * phpcs:disable
   * @I Allow plugins to implement their own purging methods
   *    type     : feature
   *    priority : normal
   *    labels   : plugin
   *    notes    : For example, to implement soft-deletion.
   * phpcs:enable
   */
  protected function doPurge(PurgeTypeInterface $purge_type): void {
    $plugin = $purge_type->getPlugin();

    foreach ($plugin->collectEntities($purge_type) as $entity) {
      $log = $plugin->getConfiguration()['log'];

      if (!$plugin->allowEntityPurge($purge_type, $entity)) {
        $log ? $this->logEntitySkip($purge_type, $entity) : NULL;
        continue;
      }

      $this->entityTypeManager
        ->getStorage($entity->getEntityTypeId())
        ->delete([$entity]);

      $log ? $this->logEntityPurge($purge_type, $entity) : NULL;
    }
  }

  /**
   * Logs a message for an entity has been deleted.
   *
   * @param \Drupal\entity_purge\Entity\PurgeTypeInterface $purge_type
   *   The purge type.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   */
  protected function logEntityPurge(
    PurgeTypeInterface $purge_type,
    EntityInterface $entity,
  ): void {
    $this->logger->info(
      '@entity_type: %entity_label (ID: @entity_id) was deleted by the %purge_type_label Entity Purge.',
      $this->buildLogContext($purge_type, $entity),
    );
  }

  /**
   * Logs a message for an entity that was not allowed to be deleted.
   *
   * @param \Drupal\entity_purge\Entity\PurgeTypeInterface $purge_type
   *   The purge type.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   */
  protected function logEntitySkip(
    PurgeTypeInterface $purge_type,
    EntityInterface $entity,
  ): void {
    $this->logger->debug(
      '@entity_type: %entity_label (ID: @entity_id) was not allowed to be deleted by the %purge_type_label Entity Purge.',
      $this->buildLogContext($purge_type, $entity),
    );
  }

  /**
   * Logs a message for an entity that was not allowed to be deleted.
   *
   * @param \Drupal\entity_purge\Entity\PurgeTypeInterface $purge_type
   *   The purge type.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   */
  protected function buildLogContext(
    PurgeTypeInterface $purge_type,
    EntityInterface $entity,
  ): array {
    $context = [
      '@entity_type' => $entity->getEntityTypeId(),
      '@entity_id' => (string) $entity->id(),
      '%entity_label' => $entity->label(),
      '%purge_type_label' => (string) $purge_type->label(),
    ];
    try {
      $context['link'] = $entity->toLink($this->t('View'))->toString();
    }
    catch (UndefinedLinkTemplateException $exception) {
    }

    return $context;
  }

}
