<?php

namespace Drupal\Tests\entity_purge\Unit\Configurator;

use Drupal\Core\Entity\EntityInterface;
use Drupal\entity_purge\Configurator\PluginBase;
use Drupal\entity_purge\Entity\PurgeTypeInterface;

/**
 * Provides tests that can be reused by all configurator plugins.
 *
 * These can be used to guarantee behaviors expected by all configurator plugins
 * after their customizations.
 */
trait PluginBaseTestTrait {

  /**
   * Tests that entities are not purged when debugging is enabled.
   *
   * @covers ::allowEntityPurge
   */
  public function testAllowEntityPurgeWithDebugging(
    string $class = PluginBase::class,
    array $extra_configuration = [],
    array $extra_arguments = [],
  ) {
    $plugin = $this->buildPlugin(
      $class,
      [
        ['debug' => TRUE] + $extra_configuration,
        'test',
        [],
      ],
      $extra_arguments,
    );

    $purge_type = $this->prophesize(PurgeTypeInterface::class);
    $entity = $this->prophesize(EntityInterface::class);

    $this->assertEquals(
      FALSE,
      $plugin->allowEntityPurge($purge_type->reveal(), $entity->reveal()),
    );
  }

  /**
   * Tests that entities are purged when debugging is disabled.
   *
   * @covers ::allowEntityPurge
   */
  public function testAllowEntityPurgeWithoutDebugging(
    string $class = PluginBase::class,
    array $extra_configuration = [],
    array $extra_arguments = [],
  ) {
    $plugin = $this->buildPlugin(
      $class,
      [
        ['debug' => FALSE] + $extra_configuration,
        'test',
        [],
      ],
      $extra_arguments,
    );

    $purge_type = $this->prophesize(PurgeTypeInterface::class);
    $entity = $this->prophesize(EntityInterface::class);

    $this->assertEquals(
      TRUE,
      $plugin->allowEntityPurge($purge_type->reveal(), $entity->reveal()),
    );
  }

  /**
   * Tests that entities are purged with the default configuration.
   *
   * @covers ::allowEntityPurge
   */
  public function testAllowEntityPurgeWithDefaults(
    string $class = PluginBase::class,
    array $extra_configuration = [],
    array $extra_arguments = [],
  ) {
    $plugin = $this->buildPlugin(
      $class,
      [
        [] + $extra_configuration,
        'test',
        [],
      ],
      $extra_arguments,
    );

    $purge_type = $this->prophesize(PurgeTypeInterface::class);
    $entity = $this->prophesize(EntityInterface::class);

    $this->assertEquals(
      TRUE,
      $plugin->allowEntityPurge($purge_type->reveal(), $entity->reveal()),
    );
  }

  /**
   * Tests the expected default configuration.
   *
   * @covers ::__construct
   */
  public function testDefaultConfiguration(
    string $class = PluginBase::class,
    array $extra_configuration = [],
    array $extra_arguments = [],
  ) {
    $plugin = $this->buildPlugin(
      $class,
      [
        [] + $extra_configuration,
        'test',
        [],
      ],
      $extra_arguments,
    );

    $configuration = $plugin->getConfiguration();

    // Debugging must be disabled by default; otherwise entities are not purged.
    $this->assertEquals(FALSE, $configuration['debug'] ?? NULL);

    // Logging must be disabled by default; otherwise excessive logging might
    // occur since this module may be used for bulk deletion.
    $this->assertEquals(FALSE, $configuration['log'] ?? NULL);

    // The scheduler must be `custom` by default; otherwise entities will be
    // purged during cron, for example. We want developers to explicitly choose
    // their scheduling.
    $this->assertEquals('custom', $configuration['scheduler'] ?? NULL);
  }

  /**
   * Returns the plugin instance to be used in these tests.
   */
  protected function buildPlugin(
    string $class,
    array $arguments,
    array $extra_arguments,
  ) {
    $arguments = array_merge(
      $arguments,
      $extra_arguments,
    );
    if ((new \ReflectionClass($class))->isAbstract()) {
      return $this->getMockForAbstractClass(
        PluginBase::class,
        $arguments,
      );
    }

    return new $class(...$arguments);
  }

}
