<?php

namespace Drupal\Tests\entity_purge\Unit;

use Drupal\entity_purge\Entity\PurgeTypeInterface;
use Drupal\entity_purge\Configurator\PluginInterface as ConfiguratorInterface;
use Drupal\entity_purge\Purger;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Link;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Tests\UnitTestCase;

use Prophecy\Argument;
use Prophecy\Argument\ArgumentsWildcard;
use Prophecy\Prophecy\ObjectProphecy;
use Psr\Log\LoggerInterface;

/**
 * @coversDefaultClass \Drupal\entity_purge\Purger
 *
 * @group contrib
 * @group entity_purge
 * @group unit
 */
class PurgerTest extends UnitTestCase {

  /**
   * Tests that the purger does not run purges of disabled types.
   *
   * @covers ::run
   */
  public function testDisabled() {
    $purge_type = $this->prophesize(PurgeTypeInterface::class);
    $purge_type
      ->status()
      ->willReturn(FALSE);

    // Test that the plugin is never called to collect the entities. This
    // ensures that the purge is not run.
    $purge_type
      ->getPlugin()
      ->shouldNotBeCalled();

    $purge_type = $purge_type->reveal();

    // The storage should never be called either.
    $entity_type_manager = $this->prophesize(EntityTypeManagerInterface::class);
    $entity_type_manager
      ->getStorage()
      ->shouldNotBeCalled();

    $entity_type_manager = $entity_type_manager->reveal();

    $purger = new Purger(
      $entity_type_manager,
      $this->prophesize(LoggerInterface::class)->reveal(),
      $this->prophesize(TranslationInterface::class)->reveal(),
    );
    $purger->run([$purge_type]);
  }

  /**
   * Tests that the purger runs multiple purges.
   *
   * @covers ::run
   */
  public function testMultiple() {
    $purge_types = \array_map(
      function () {
        $purge_type = $this->prophesize(PurgeTypeInterface::class);
        $purge_type
          ->status()
          ->shouldBeCalledOnce()
          ->willReturn(FALSE);

        return $purge_type->reveal();
      },
      [0, 1],
    );

    $purger = new Purger(
      $this->prophesize(EntityTypeManagerInterface::class)->reveal(),
      $this->prophesize(LoggerInterface::class)->reveal(),
      $this->prophesize(TranslationInterface::class)->reveal(),
    );
    $purger->run($purge_types);
  }

  /**
   * Tests that no entities are deleted when none are collected by the plugin.
   *
   * @covers ::run
   */
  public function testEmptyCollection() {
    $purge_type = $this->prophesize(PurgeTypeInterface::class);
    $purge_type
      ->status()
      ->willReturn(TRUE);

    $configurator = $this->prophesize(ConfiguratorInterface::class);
    $configurator
      ->collectEntities($purge_type)
      ->willReturn([]);

    $configurator = $configurator->reveal();

    $purge_type
      ->getPlugin()
      ->willReturn($configurator);

    $purge_type = $purge_type->reveal();

    $entity_type_manager = $this->prophesize(EntityTypeManagerInterface::class);
    $entity_type_manager
      ->getStorage(Argument::any())
      ->shouldNotBeCalled();

    $entity_type_manager = $entity_type_manager->reveal();

    $purger = new Purger(
      $entity_type_manager,
      $this->prophesize(LoggerInterface::class)->reveal(),
      $this->prophesize(TranslationInterface::class)->reveal(),
    );
    $purger->run([$purge_type]);
  }

  /**
   * Tests that entities are not deleted when the plugin disallows.
   *
   * @covers ::run
   */
  public function testDisallowedEntityWithoutLogging() {
    $purge_type = $this->prophesize(PurgeTypeInterface::class);
    $purge_type
      ->status()
      ->willReturn(TRUE);

    $entity = $this->prophesize(EntityInterface::class);
    $entity
      ->delete()
      ->shouldNotBeCalled();
    $entity = $entity->reveal();

    $configurator = $this->prophesize(ConfiguratorInterface::class);
    $configurator
      ->collectEntities($purge_type)
      ->willReturn([$entity]);
    $configurator
      ->getConfiguration()
      ->willReturn(['log' => FALSE]);
    $configurator
      ->allowEntityPurge($purge_type, $entity)
      ->willReturn(FALSE);

    $configurator = $configurator->reveal();

    $purge_type
      ->getPlugin()
      ->willReturn($configurator);

    $purge_type = $purge_type->reveal();

    $entity_type_manager = $this->prophesize(EntityTypeManagerInterface::class);
    $entity_type_manager
      ->getStorage(Argument::any())
      ->shouldNotBeCalled();

    $entity_type_manager = $entity_type_manager->reveal();

    $logger = $this->prophesize(LoggerInterface::class);

    $purger = new Purger(
      $entity_type_manager,
      $logger->reveal(),
      $this->prophesize(TranslationInterface::class)->reveal(),
    );
    $purger->run([$purge_type]);

    $this->assertNoMethodHasBeenCalled($logger, LoggerInterface::class);
  }

  /**
   * Tests that a message is logged when an entity deletion is disallowed.
   *
   * @covers ::run
   */
  public function testDisallowedEntityWithLogging() {
    $purge_type = $this->prophesize(PurgeTypeInterface::class);
    $purge_type
      ->status()
      ->willReturn(TRUE);
    $purge_type
      ->label()
      ->willReturn('Purge type');

    $entity = $this->prophesize(EntityInterface::class);
    $entity
      ->getEntityTypeId()
      ->willReturn('node');
    $entity
      ->id()
      ->willReturn('1');
    $entity
      ->label()
      ->willReturn('Content item');

    $link = $this->prophesize(Link::class);
    $link
      ->toString()
      ->willReturn('Link');
    $link->reveal();
    $entity
      ->toLink(Argument::any())
      ->willReturn($link->reveal());

    $configurator = $this->prophesize(ConfiguratorInterface::class);
    $configurator
      ->collectEntities($purge_type)
      ->willReturn([$entity->reveal()]);
    $configurator
      ->getConfiguration()
      ->willReturn(['log' => TRUE]);
    $configurator
      ->allowEntityPurge($purge_type, $entity->reveal())
      ->willReturn(FALSE);

    $purge_type
      ->getPlugin()
      ->willReturn($configurator->reveal());

    $entity_type_manager = $this->prophesize(EntityTypeManagerInterface::class);
    $entity_type_manager
      ->getStorage(Argument::any())
      ->shouldNotBeCalled();

    $logger = $this->prophesize(LoggerInterface::class);
    $logger
      ->debug(Argument::any(), Argument::any())
      ->shouldBeCalledOnce();

    $purger = new Purger(
      $entity_type_manager->reveal(),
      $logger->reveal(),
      $this->prophesize(TranslationInterface::class)->reveal(),
    );
    $purger->run([$purge_type->reveal()]);
  }

  /**
   * Tests that entities are deleted when collected and allowed by the plugin.
   *
   * @covers ::run
   */
  public function testAllowedEntityWithoutLogging() {
    $purge_type = $this->prophesize(PurgeTypeInterface::class);
    $purge_type
      ->status()
      ->willReturn(TRUE);

    $entity = $this->prophesize(EntityInterface::class)->reveal();

    $configurator = $this->prophesize(ConfiguratorInterface::class);
    $configurator
      ->collectEntities($purge_type)
      ->willReturn([$entity]);
    $configurator
      ->getConfiguration()
      ->willReturn(['log' => FALSE]);
    $configurator
      ->allowEntityPurge($purge_type, $entity)
      ->willReturn(TRUE);

    $purge_type
      ->getPlugin()
      ->willReturn($configurator->reveal());

    $storage = $this->prophesize(EntityStorageInterface::class);
    $storage
      ->delete([$entity])
      ->shouldBeCalledOnce();
    $entity_type_manager = $this->prophesize(EntityTypeManagerInterface::class);
    $entity_type_manager
      ->getStorage(Argument::any())
      ->willReturn($storage->reveal());

    $logger = $this->prophesize(LoggerInterface::class);

    $purger = new Purger(
      $entity_type_manager->reveal(),
      $logger->reveal(),
      $this->prophesize(TranslationInterface::class)->reveal(),
    );
    $purger->run([$purge_type->reveal()]);

    $this->assertNoMethodHasBeenCalled($logger, LoggerInterface::class);
  }

  /**
   * Tests that a message is logged when an entity is deleted.
   *
   * @covers ::run
   */
  public function testAllowedEntityWithLogging() {
    $purge_type = $this->prophesize(PurgeTypeInterface::class);
    $purge_type
      ->status()
      ->willReturn(TRUE);
    $purge_type
      ->label()
      ->willReturn('Purge type');

    $entity = $this->prophesize(EntityInterface::class);
    $entity
      ->getEntityTypeId()
      ->willReturn('node');
    $entity
      ->id()
      ->willReturn('1');
    $entity
      ->label()
      ->willReturn('Content item');

    $link = $this->prophesize(Link::class);
    $link
      ->toString()
      ->willReturn('Link');
    $link->reveal();

    $entity
      ->toLink(Argument::any())
      ->willReturn($link->reveal());

    $configurator = $this->prophesize(ConfiguratorInterface::class);
    $configurator
      ->collectEntities($purge_type)
      ->willReturn([$entity->reveal()]);
    $configurator
      ->getConfiguration()
      ->willReturn(['log' => TRUE]);
    $configurator
      ->allowEntityPurge($purge_type, $entity->reveal())
      ->willReturn(TRUE);

    $purge_type
      ->getPlugin()
      ->willReturn($configurator->reveal());

    $storage = $this->prophesize(EntityStorageInterface::class);
    $storage
      ->delete([$entity])
      ->shouldBeCalledOnce();
    $entity_type_manager = $this->prophesize(EntityTypeManagerInterface::class);
    $entity_type_manager
      ->getStorage(Argument::any())
      ->willReturn($storage->reveal());

    $logger = $this->prophesize(LoggerInterface::class);
    $logger
      ->info(Argument::any(), Argument::any())
      ->shouldBeCalledOnce();

    $purger = new Purger(
      $entity_type_manager->reveal(),
      $logger->reveal(),
      $this->prophesize(TranslationInterface::class)->reveal(),
    );
    $purger->run([$purge_type->reveal()]);
  }

  /**
   * Asserts that no method has been called on the given prophesized object.
   *
   * @param \Prophecy\Prophecy\ObjectProphecy $prophecy
   *   The prophesized object.
   * @param string $class_name
   *   The class of the original object/interface prophesized.
   */
  protected function assertNoMethodHasBeenCalled(
    ObjectProphecy $prophecy,
    string $class_name,
  ): void {
    $methods = get_class_methods($class_name);

    $calls = [];
    foreach ($methods as $method) {
      $reflection_method = new \ReflectionMethod($class_name, $method);
      $arguments = new ArgumentsWildcard(array_fill(
        0,
        count($reflection_method->getParameters()),
        Argument::any(),
      ));
      $calls = array_merge(
        $calls,
        $prophecy->findProphecyMethodCalls(
          $method,
          $arguments,
        ),
      );
    }

    $this->assertEmpty($calls);
  }

}
